"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const log_1 = __importDefault(require("./log"));
const provideRoutes_1 = __importDefault(require("./provideRoutes"));
const multer_1 = __importDefault(require("multer"));
let app;
let server;
let previousPort = 0;
function close(callback) {
    server && server.close(callback);
    server = undefined;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function requireDefault(file) {
    // eslint-disable-next-line import/no-dynamic-require, global-require, @typescript-eslint/no-var-requires
    const m = require(file.path);
    const arr = [];
    function moduleEachWrapper(usedApp) {
        arr.forEach(f => f(usedApp));
    }
    Object.keys(m).forEach(key => {
        const f = m[key];
        if (typeof f !== "function") {
            log_1.default.error(`Wrong 'export ${key} = ${f}' from ${file.rootName}: expected exporting only functions`);
        }
        else {
            arr.push(f);
        }
    });
    return moduleEachWrapper;
}
let isFirstStart = true;
function mockServer(attachedFileNames, options, listenCallback) {
    log_1.default.debug(!server ? "starting" : "re-starting");
    /*
     *  self-destroying
     */
    if (isFirstStart) {
        process.on("SIGINT", () => close()); // handle termination by Ctrl+C
        process.on("beforeExit", () => close());
        isFirstStart = false;
    }
    close();
    app = express_1.default();
    app.set("json spaces", 2); // prettify json-response
    app.use(express_1.default.json());
    app.use(express_1.default.urlencoded({ extended: true })); // support form-urlencoded
    app.use(express_1.default.text()); // support ordinary text
    app.use(multer_1.default().none()); // support multipart/form-data
    options.before && app.use(options.before);
    // logMiddleware
    if (options.logRequests || options.logResponses) {
        app.use((req, res, next) => {
            if (options.logRequests) {
                log_1.default.info(`Got request: ${req.method}`, req.url, {
                    httpVersion: req.httpVersion,
                    headers: req.headers,
                    params: req.params,
                    cookies: req.cookies
                });
            }
            if (!options.logResponses) {
                next();
                return;
            }
            const oldWrite = res.write;
            const oldEnd = res.end;
            const chunks = [];
            // @ts-ignore
            res.write = function hookWrite(chunk) {
                chunks.push(chunk);
                // @ts-ignore
                return oldWrite.apply(res, arguments);
            };
            // @ts-ignore
            res.end = function hookEnd(chunk) {
                if (chunk) {
                    chunks.push(chunk);
                }
                // @ts-ignore
                oldEnd.apply(res, arguments);
            };
            res.once("finish", () => {
                const headers = res.getHeaders();
                const contentType = headers["content-type"] || "";
                const isJson = contentType.startsWith("application/json");
                const isText = contentType.startsWith("text/");
                // eslint-disable-next-line @typescript-eslint/ban-types
                let body;
                try {
                    if (chunks.length) {
                        const isBufferArray = !chunks.some(v => !v || !(v instanceof Buffer || v instanceof Uint8Array));
                        // @ts-ignore
                        if (isJson) {
                            let str = "";
                            if (isBufferArray) {
                                str = Buffer.concat(chunks).toString("utf8");
                            }
                            else {
                                str = chunks.join("\n");
                            }
                            body = str && JSON.parse(str);
                        }
                        else if (isText) {
                            body = chunks.join("\n");
                        }
                        else if (isBufferArray) {
                            body = "[byteArray]";
                        }
                        else {
                            body = chunks;
                        }
                    }
                }
                catch (ex) {
                    log_1.default.error("", ex);
                }
                log_1.default.info(`Sent response for ${req.method}`, req.url, {
                    headers: Object.assign({}, headers),
                    statusCode: res.statusCode,
                    statusMessage: res.statusMessage,
                    body
                });
            });
            next();
        });
    }
    const mockedInfoPath = "/";
    app.get(mockedInfoPath, (_req, res) => {
        try {
            const routes = provideRoutes_1.default(app, mockedInfoPath);
            const html = fs_1.default
                .readFileSync(path_1.default.resolve(__dirname, "../public/index.html"), "utf8")
                .replace("{routes}", JSON.stringify(routes));
            res.send(html);
        }
        catch (ex) {
            res.send("Mock server is ready");
            log_1.default.error("Exception in provideRoutes()", ex);
        }
    });
    // provides favicon only in case if request to this server directly but not to via ParentProxyServer
    app.get("/favicon.ico", (req, res, next) => {
        const hostHeader = req.header("host");
        // in this case previousPort == currentPort
        if (hostHeader && hostHeader.includes(`:${previousPort}`)) {
            res.sendFile(require.resolve("../public/favicon.ico"));
        }
        else {
            next();
        }
    });
    function listen(port) {
        server = app
            .listen(port, function gotPort() {
            if (port !== previousPort) {
                log_1.default.info("Started at", `http://localhost:${port}/`);
                previousPort = port;
            }
            else {
                log_1.default.debug("Started at", `http://localhost:${port}/`);
            }
            listenCallback && listenCallback(port, server);
        })
            .on("error", function listenErrorCallback(err) {
            if (err.code === "EADDRINUSE" || err.code === "EACCES") {
                listen(port + 1);
            }
            else {
                log_1.default.error("", err);
            }
        });
    }
    try {
        if (attachedFileNames.length === 0) {
            log_1.default.error("There are no rootFiles");
        }
        else {
            log_1.default.debug("import rootFiles:", "", attachedFileNames.map(v => v.path));
            attachedFileNames.forEach(v => requireDefault(v)(app));
        }
        listen(options.port);
    }
    catch (ex) {
        log_1.default.error("Exception during attaching node-modules", ex);
    }
    return app;
}
exports.default = mockServer;
